﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the display resolution.
    /// </summary>
    public struct Resolution
    {
        /// <summary>
        /// Returns the resolution of the primary screen or sets it.
        /// </summary>
        public static Resolution CurrentResolution
        {
            get
            {
                Resolution res = new Resolution();
                res.Width = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width;
                res.Height = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height;
                res.ColorDepth = System.Windows.Forms.Screen.PrimaryScreen.BitsPerPixel;
                return res;
            }
            set
            {
                Win32Api.DEVMODE dm = new Win32Api.DEVMODE();
                if (!Win32Api.EnumDisplaySettings(null, Win32Api.ENUM_CURRENT_SETTINGS, ref dm))
                {
                    dm.dmPelsWidth = value.Width;
                    dm.dmPelsHeight = value.Height;
                    dm.dmBitsPerPel = value.ColorDepth;

                    Win32Api.DISP_CHANGE result = Win32Api.ChangeDisplaySettingsEx(null, ref dm, IntPtr.Zero, Win32Api.CDS_TEST, IntPtr.Zero);
                    if (result == Win32Api.DISP_CHANGE.Failed)
                    {
                        MessageBox.Show("Process could not be executed due to compatiblity issues.", "Compatiblity issues", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    else
                    {
                        result = Win32Api.ChangeDisplaySettingsEx(null, ref dm, IntPtr.Zero, TweakAPI.Win32Api.CDS_UPDATEREGISTRY, IntPtr.Zero);
                        switch (result)
                        {
                            case Win32Api.DISP_CHANGE.Successful:
                                MessageBox.Show("Resolution was changed successfully.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                break;
                            case Win32Api.DISP_CHANGE.Restart:
                                MessageBox.Show("For the changes to take effect, you must restart Windows", "Restart required", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                break;
                            case Win32Api.DISP_CHANGE.NotUpdated:
                                MessageBox.Show("The changes have not been taken over yet.", "Not taken over yet", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                break;
                            case Win32Api.DISP_CHANGE.BadParam:
                                MessageBox.Show("Resolution could not be changed, because of bad params.", "Bad params", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                break;
                            case Win32Api.DISP_CHANGE.BadMode:
                                MessageBox.Show("Resolution could not be changed,  because of not supported values.", "Not supported values.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                break;
                            case Win32Api.DISP_CHANGE.BadFlags:
                                MessageBox.Show("Resolution could not be changed, because of invalid flags.", "Invalid flags", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                break;
                            case Win32Api.DISP_CHANGE.BadDualView:
                                MessageBox.Show("Resolution could not be changed, because multiple monitors are connected.", "More than one monitor", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                break;
                            case Win32Api.DISP_CHANGE.Failed:
                                MessageBox.Show("Resolution could not be changed due to unknown problems.", "Unknown problems", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                break;
                        }
                    }
                }
                else
                {
                    MessageBox.Show("Resolution could not be determined.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Returns a <see cref="Resolution"/>-Array containing the resolutions which are supported by the display.
        /// </summary>
        /// <returns></returns>
        public static Resolution[] EnumerateResolutions()
        {
            List<Resolution> resolutions = new List<Resolution>();
            bool lResult = true;
            int i = 0;
            Win32Api.DEVMODE dm = new Win32Api.DEVMODE();
            do
            {
                lResult = Win32Api.EnumDisplaySettings(null, i, ref dm);
                Resolution res = new Resolution();
                res.Width = dm.dmPelsWidth;
                res.Height = dm.dmPelsHeight;
                if (!(resolutions.Contains(res)) && (dm.dmBitsPerPel == 32))
                {
                    resolutions.Add(res);
                }
                i += 1;
            }
            while (lResult == true);
            return resolutions.ToArray();
        }

        /// <summary>
        /// The width.
        /// </summary>
        public int Width { get; set; }
        /// <summary>
        /// The height.
        /// </summary>
        public int Height { get; set; }
        /// <summary>
        /// The color depth.
        /// </summary>
        public int ColorDepth { get; set; }

        /// <summary>
        /// Converts the current <see cref="Resolution"/> object to a new <see cref="string"/> object. 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return Width.ToString() + "x" + Height.ToString();
        }

        /// <summary>
        /// Creates a new <see cref="Resolution"/> object from the specified <see cref="Rectangle"/> object.
        /// </summary>
        /// <param name="r">The rectangle.</param>
        /// <param name="colorDepth">Optional. The color depth.</param>
        /// <returns></returns>
        public static Resolution FromRectangle(Rectangle r, int colorDepth = 32)
        {
            Resolution res = new Resolution();
            res.Width = r.Width;
            res.Height = r.Height;
            res.ColorDepth = colorDepth;
            return res;
        }
    }
}
