﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Management;
using Microsoft.Win32;
using System.Drawing;
using System.Globalization;
using System.Text.RegularExpressions;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the Windows version.
    /// </summary>
    public static class Windows
    {
        private static ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_OperatingSystem");
        private static RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", 
            RegistryKeyPermissionCheck.ReadSubTree);

        /// <summary>
        /// Returns the name of the operating system.
        /// </summary>
        public static string Name
        {
            get
            {
                return regKey.GetValue("ProductName").ToString();                
            }
        }

        /// <summary>
        /// Returns the version of the operating system.
        /// </summary>
        public static string Version
        {
            get
            {
                if (Name.Contains("Windows 10"))
                {
                    int build = int.Parse(regKey.GetValue("CurrentBuild").ToString());
                    if (build < 10586)
                    {
                        int major = int.Parse(regKey.GetValue("CurrentMajorVersionNumber").ToString());
                        int minor = int.Parse(regKey.GetValue("CurrentMinorVersionNumber").ToString());
                        return (major.ToString() + "." + minor.ToString() + " [Build " + build + "]");
                    }
                    else
                    {
                        string version = regKey.GetValue("ReleaseId").ToString();
                        int ubr = int.Parse(regKey.GetValue("UBR").ToString());
                        return (version + " [Build " + build + "." + ubr + "]");
                    }
                }
                else
                { return regKey.GetValue("CurrentVersion").ToString() + " [Build " + regKey.GetValue("CurrentBuild").ToString() + "]"; }
            }
        }

        /// <summary>
        /// Returns the platform caption of your operating system.
        /// </summary>
        public static string Platform
        {
            get
            {
                return Environment.OSVersion.Platform.ToString();
            }
        }

        /// <summary>
        /// Returns the registered manufacturer of your operating system.
        /// </summary>
        public static string Manufacturer
        {
            get
            {
                string manu = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    manu = queryObj["Manufacturer"].ToString();
                }
                return manu;
            }
        }

        /// <summary>
        /// Returns the installed service pack.
        /// </summary>
        public static string ServicePack
        {
            get
            {
                return Environment.OSVersion.ServicePack;
            }
        }

        /// <summary>
        /// Returns the codename which was used in development period.
        /// </summary>
        public static string Codename
        {
            get
            {
                string cn = null;
                if (Version.Contains("5.1"))
                { cn = "Whistler"; }
                else if (Version.Contains("5.2"))
                { cn = "Whistler Server"; }
                else if (Version.Contains("6.0"))
                { 
                    if (Name.Contains("Vista"))
                    { cn = "Longhorn"; }
                    else if (Name.Contains("2008"))
                    { cn = "Longhorn Server"; }
                }
                else if (Version.Contains("6.1"))
                { cn = "Blackcomb, Vienna"; }
                else if (Version.Contains("6.2") || Version.Contains("6.3"))
                { cn = "Mystic, Orient"; }
                else if (Version.Contains("6.4") || Version.Contains("10.0") || Name.Contains("Windows 10"))
                { cn = "Threshold"; }
                return cn;
            }
        }

        /// <summary>
        /// Encodes the product key of your operating system.
        /// </summary>
        public static string ProductKey
        {
            get
            {
                IList<byte> digitalProductId = null;
                {
                    RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", 
                        RegistryKeyPermissionCheck.ReadSubTree);
                    if (regKey != null)
                    {
                        digitalProductId = regKey.GetValue("DigitalProductId")
                            as byte[];
                        regKey.Close();
                    }
                    else return null;
                }

                int keyStartIndex = 52;
                int keyEndIndex = keyStartIndex + 15;

                const int numLetters = 24;
                char[] digits = new[]
                { 'B', 'C', 'D', 'F', 'G', 'H', 'J', 'K', 'M', 'P', 'Q', 'R', 'T', 'V', 'W', 'X', 'Y', '2', '3', '4', '6', '7', '8', '9' };

                int containsN = (digitalProductId[keyStartIndex + 14] >> 3) & 1;
                digitalProductId[keyStartIndex + 14] = (byte)((digitalProductId[keyStartIndex + 14] & 0xF7) | ((containsN & 2) << 2));

                const int decodeLength = 29;

                const int decodeStringLength = 15;

                char[] decodedChars = new char[decodeLength];

                List<byte> hexPid = new List<byte>();
                for (int i = keyStartIndex; i <= keyEndIndex; i++)
                {
                    hexPid.Add(digitalProductId[i]);
                }
                for (int i = decodeLength - 1; i >= 0; i--)
                {
                    if ((i + 1) % 6 == 0)
                    {
                        decodedChars[i] = '-';
                    }
                    else
                    {
                        int digitMapIndex = 0;
                        for (int j = decodeStringLength - 1; j >= 0; j--)
                        {
                            int byteValue = (digitMapIndex << 8) | hexPid[j];
                            hexPid[j] = (byte)(byteValue / numLetters);
                            digitMapIndex = byteValue % numLetters;
                            decodedChars[i] = digits[digitMapIndex];
                        }
                    }
                }
                if (containsN != 0)
                {
                    int firstLetterIndex = 0;
                    for (int index = 0; index < numLetters; index++)
                    {
                        if (decodedChars[0] != digits[index]) continue;
                        firstLetterIndex = index;
                        break;
                    }
                    string keyWithN = new string(decodedChars);

                    keyWithN = keyWithN.Replace("-", string.Empty).Remove(0, 1);
                    keyWithN = keyWithN.Substring(0, firstLetterIndex) + "N" +
                                    keyWithN.Remove(0, firstLetterIndex);
                    keyWithN = keyWithN.Substring(0, 5) + "-" + keyWithN.Substring(5, 5) + "-" +
                                    keyWithN.Substring(10, 5) + "-" + keyWithN.Substring(15, 5) + "-" +
                                    keyWithN.Substring(20, 5);
                    
                    return keyWithN;
                }
                return new string(decodedChars);
            }
        }

        /// <summary>
        /// Returns the architecture of Windows.
        /// </summary>
        public static Helper.Architecture Architecture
        {
            get
            {
                Helper.Architecture arch = Helper.Architecture.x64;
                if (Environment.Is64BitOperatingSystem)
                {
                    arch = Helper.Architecture.x64;
                }
                else
                {
                    arch = Helper.Architecture.x86;
                }
                return arch;
            }
        }

        /// <summary>
        /// Returns the current system language.
        /// </summary>
        public static string Language
        {
            get
            {
                CultureInfo ci = CultureInfo.InstalledUICulture;
                string lang = ci.DisplayName;
                return lang;
            }
        }

        /// <summary>
        /// Returns the path, where the Windows files are located.
        /// </summary>
        public static string SystemDirectory
        {
            get
            {
                return Environment.GetFolderPath(Environment.SpecialFolder.System);
            }
        }

        /// <summary>
        /// Returns the product ID of Windows.
        /// </summary>
        public static string ProductID
        {
            get
            {
                string prodid = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    prodid = queryObj["SerialNumber"].ToString();
                }
                return prodid;
            }
        } 
    }
}
